@@ -19,44 +19,27 @@ import copy
 import functools
 import uuid
 
-import six
+from oslo_config import cfg
+from oslo_log import log
 from six.moves import urllib
 
 from keystone.assignment import schema
 from keystone.common import controller
 from keystone.common import dependency
 from keystone.common import validation
-from keystone import config
 from keystone import exception
-from keystone.i18n import _
+from keystone.i18n import _, _LW
 from keystone.models import token_model
-from keystone.openstack.common import log
+from keystone import notifications
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 
 @dependency.requires('assignment_api', 'identity_api', 'token_provider_api')
-class Tenant(controller.V2Controller):
-
-    @controller.v2_deprecated
-    def get_all_projects(self, context, **kw):
-        """Gets a list of all tenants for an admin user."""
-        if 'name' in context['query_string']:
-            return self.get_project_by_name(
-                context, context['query_string'].get('name'))
-
-        self.assert_admin(context)
-        tenant_refs = self.assignment_api.list_projects_in_domain(
-            CONF.identity.default_domain_id)
-        for tenant_ref in tenant_refs:
-            tenant_ref = self.filter_domain_id(tenant_ref)
-        params = {
-            'limit': context['query_string'].get('limit'),
-            'marker': context['query_string'].get('marker'),
-        }
-        return self._format_project_list(tenant_refs, **params)
+class TenantAssignment(controller.V2Controller):
+    """The V2 Project APIs that are processing assignments."""
 
     @controller.v2_deprecated
     def get_projects_for_token(self, context, **kw):
@@ -74,65 +57,18 @@ class Tenant(controller.V2Controller):
             token_ref = token_model.KeystoneToken(token_id=context['token_id'],
                                                   token_data=token_data)
         except exception.NotFound as e:
-            LOG.warning(_('Authentication failed: %s'), e)
+            LOG.warning(_LW('Authentication failed: %s'), e)
             raise exception.Unauthorized(e)
 
         tenant_refs = (
             self.assignment_api.list_projects_for_user(token_ref.user_id))
-        tenant_refs = [self.filter_domain_id(ref) for ref in tenant_refs
+        tenant_refs = [self.v3_to_v2_project(ref) for ref in tenant_refs
                        if ref['domain_id'] == CONF.identity.default_domain_id]
         params = {
             'limit': context['query_string'].get('limit'),
             'marker': context['query_string'].get('marker'),
         }
-        return self._format_project_list(tenant_refs, **params)
-
-    @controller.v2_deprecated
-    def get_project(self, context, tenant_id):
-        # TODO(termie): this stuff should probably be moved to middleware
-        self.assert_admin(context)
-        ref = self.assignment_api.get_project(tenant_id)
-        return {'tenant': self.filter_domain_id(ref)}
-
-    @controller.v2_deprecated
-    def get_project_by_name(self, context, tenant_name):
-        self.assert_admin(context)
-        ref = self.assignment_api.get_project_by_name(
-            tenant_name, CONF.identity.default_domain_id)
-        return {'tenant': self.filter_domain_id(ref)}
-
-    # CRUD Extension
-    @controller.v2_deprecated
-    def create_project(self, context, tenant):
-        tenant_ref = self._normalize_dict(tenant)
-
-        if 'name' not in tenant_ref or not tenant_ref['name']:
-            msg = _('Name field is required and cannot be empty')
-            raise exception.ValidationError(message=msg)
-
-        self.assert_admin(context)
-        tenant_ref['id'] = tenant_ref.get('id', uuid.uuid4().hex)
-        tenant = self.assignment_api.create_project(
-            tenant_ref['id'],
-            self._normalize_domain_id(context, tenant_ref))
-        return {'tenant': self.filter_domain_id(tenant)}
-
-    @controller.v2_deprecated
-    def update_project(self, context, tenant_id, tenant):
-        self.assert_admin(context)
-        # Remove domain_id if specified - a v2 api caller should not
-        # be specifying that
-        clean_tenant = tenant.copy()
-        clean_tenant.pop('domain_id', None)
-
-        tenant_ref = self.assignment_api.update_project(
-            tenant_id, clean_tenant)
-        return {'tenant': tenant_ref}
-
-    @controller.v2_deprecated
-    def delete_project(self, context, tenant_id):
-        self.assert_admin(context)
-        self.assignment_api.delete_project(tenant_id)
+        return self.format_project_list(tenant_refs, **params)
 
     @controller.v2_deprecated
     def get_project_users(self, context, tenant_id, **kw):
@@ -152,43 +88,51 @@ class Tenant(controller.V2Controller):
                 user_refs.append(self.v3_to_v2_user(user_ref))
         return {'users': user_refs}
 
-    def _format_project_list(self, tenant_refs, **kwargs):
-        marker = kwargs.get('marker')
-        first_index = 0
-        if marker is not None:
-            for (marker_index, tenant) in enumerate(tenant_refs):
-                if tenant['id'] == marker:
-                    # we start pagination after the marker
-                    first_index = marker_index + 1
-                    break
-            else:
-                msg = _('Marker could not be found')
-                raise exception.ValidationError(message=msg)
 
-        limit = kwargs.get('limit')
-        last_index = None
-        if limit is not None:
-            try:
-                limit = int(limit)
-                if limit < 0:
-                    raise AssertionError()
-            except (ValueError, AssertionError):
-                msg = _('Invalid limit value')
-                raise exception.ValidationError(message=msg)
-            last_index = first_index + limit
-
-        tenant_refs = tenant_refs[first_index:last_index]
-
-        for x in tenant_refs:
-            if 'enabled' not in x:
-                x['enabled'] = True
-        o = {'tenants': tenant_refs,
-             'tenants_links': []}
-        return o
-
-
-@dependency.requires('assignment_api')
+@dependency.requires('assignment_api', 'role_api')
 class Role(controller.V2Controller):
+    """The Role management APIs."""
+
+    @controller.v2_deprecated
+    def get_role(self, context, role_id):
+        self.assert_admin(context)
+        return {'role': self.role_api.get_role(role_id)}
+
+    @controller.v2_deprecated
+    def create_role(self, context, role):
+        role = self._normalize_dict(role)
+        self.assert_admin(context)
+
+        if 'name' not in role or not role['name']:
+            msg = _('Name field is required and cannot be empty')
+            raise exception.ValidationError(message=msg)
+
+        if role['name'] == CONF.member_role_name:
+            # Use the configured member role ID when creating the configured
+            # member role name. This avoids the potential of creating a
+            # "member" role with an unexpected ID.
+            role_id = CONF.member_role_id
+        else:
+            role_id = uuid.uuid4().hex
+
+        role['id'] = role_id
+        role_ref = self.role_api.create_role(role_id, role)
+        return {'role': role_ref}
+
+    @controller.v2_deprecated
+    def delete_role(self, context, role_id):
+        self.assert_admin(context)
+        self.role_api.delete_role(role_id)
+
+    @controller.v2_deprecated
+    def get_roles(self, context):
+        self.assert_admin(context)
+        return {'roles': self.role_api.list_roles()}
+
+
+@dependency.requires('assignment_api', 'resource_api', 'role_api')
+class RoleAssignmentV2(controller.V2Controller):
+    """The V2 Role APIs that are processing assignments."""
 
     # COMPAT(essex-3)
     @controller.v2_deprecated
@@ -200,45 +144,11 @@ class Role(controller.V2Controller):
 
         """
         self.assert_admin(context)
-        if tenant_id is None:
-            raise exception.NotImplemented(message='User roles not supported: '
-                                                   'tenant ID required')
-
         roles = self.assignment_api.get_roles_for_user_and_project(
             user_id, tenant_id)
-        return {'roles': [self.assignment_api.get_role(x)
+        return {'roles': [self.role_api.get_role(x)
                           for x in roles]}
 
-    # CRUD extension
-    @controller.v2_deprecated
-    def get_role(self, context, role_id):
-        self.assert_admin(context)
-        return {'role': self.assignment_api.get_role(role_id)}
-
-    @controller.v2_deprecated
-    def create_role(self, context, role):
-        role = self._normalize_dict(role)
-        self.assert_admin(context)
-
-        if 'name' not in role or not role['name']:
-            msg = _('Name field is required and cannot be empty')
-            raise exception.ValidationError(message=msg)
-
-        role_id = uuid.uuid4().hex
-        role['id'] = role_id
-        role_ref = self.assignment_api.create_role(role_id, role)
-        return {'role': role_ref}
-
-    @controller.v2_deprecated
-    def delete_role(self, context, role_id):
-        self.assert_admin(context)
-        self.assignment_api.delete_role(role_id)
-
-    @controller.v2_deprecated
-    def get_roles(self, context):
-        self.assert_admin(context)
-        return {'roles': self.assignment_api.list_roles()}
-
     @controller.v2_deprecated
     def add_role_to_user(self, context, user_id, role_id, tenant_id=None):
         """Add a role to a user and tenant pair.
@@ -255,7 +165,7 @@ class Role(controller.V2Controller):
         self.assignment_api.add_role_to_user_and_project(
             user_id, tenant_id, role_id)
 
-        role_ref = self.assignment_api.get_role(role_id)
+        role_ref = self.role_api.get_role(role_id)
         return {'role': role_ref}
 
     @controller.v2_deprecated
@@ -321,7 +231,7 @@ class Role(controller.V2Controller):
         self.assignment_api.add_role_to_user_and_project(
             user_id, tenant_id, role_id)
 
-        role_ref = self.assignment_api.get_role(role_id)
+        role_ref = self.role_api.get_role(role_id)
         return {'role': role_ref}
 
     # COMPAT(diablo): CRUD extension
@@ -346,143 +256,106 @@ class Role(controller.V2Controller):
             user_id, tenant_id, role_id)
 
 
-@dependency.requires('assignment_api')
-class DomainV3(controller.V3Controller):
-    collection_name = 'domains'
-    member_name = 'domain'
+@dependency.requires('assignment_api', 'resource_api')
+class ProjectAssignmentV3(controller.V3Controller):
+    """The V3 Project APIs that are processing assignments."""
 
-    def __init__(self):
-        super(DomainV3, self).__init__()
-        self.get_member_from_driver = self.assignment_api.get_domain
-
-    @controller.protected()
-    @validation.validated(schema.domain_create, 'domain')
-    def create_domain(self, context, domain):
-        ref = self._assign_unique_id(self._normalize_dict(domain))
-        ref = self.assignment_api.create_domain(ref['id'], ref)
-        return DomainV3.wrap_member(context, ref)
-
-    @controller.filterprotected('enabled', 'name')
-    def list_domains(self, context, filters):
-        hints = DomainV3.build_driver_hints(context, filters)
-        refs = self.assignment_api.list_domains(hints=hints)
-        return DomainV3.wrap_collection(context, refs, hints=hints)
-
-    @controller.protected()
-    def get_domain(self, context, domain_id):
-        ref = self.assignment_api.get_domain(domain_id)
-        return DomainV3.wrap_member(context, ref)
-
-    @controller.protected()
-    @validation.validated(schema.domain_update, 'domain')
-    def update_domain(self, context, domain_id, domain):
-        self._require_matching_id(domain_id, domain)
-        ref = self.assignment_api.update_domain(domain_id, domain)
-        return DomainV3.wrap_member(context, ref)
-
-    @controller.protected()
-    def delete_domain(self, context, domain_id):
-        return self.assignment_api.delete_domain(domain_id)
-
-
-@dependency.requires('assignment_api')
-class ProjectV3(controller.V3Controller):
     collection_name = 'projects'
     member_name = 'project'
 
     def __init__(self):
-        super(ProjectV3, self).__init__()
-        self.get_member_from_driver = self.assignment_api.get_project
-
-    @controller.protected()
-    @validation.validated(schema.project_create, 'project')
-    def create_project(self, context, project):
-        ref = self._assign_unique_id(self._normalize_dict(project))
-        ref = self._normalize_domain_id(context, ref)
-        ref = self.assignment_api.create_project(ref['id'], ref)
-        return ProjectV3.wrap_member(context, ref)
-
-    @controller.filterprotected('domain_id', 'enabled', 'name')
-    def list_projects(self, context, filters):
-        hints = ProjectV3.build_driver_hints(context, filters)
-        refs = self.assignment_api.list_projects(hints=hints)
-        return ProjectV3.wrap_collection(context, refs, hints=hints)
+        super(ProjectAssignmentV3, self).__init__()
+        self.get_member_from_driver = self.resource_api.get_project
 
     @controller.filterprotected('enabled', 'name')
     def list_user_projects(self, context, filters, user_id):
-        hints = ProjectV3.build_driver_hints(context, filters)
+        hints = ProjectAssignmentV3.build_driver_hints(context, filters)
         refs = self.assignment_api.list_projects_for_user(user_id,
                                                           hints=hints)
-        return ProjectV3.wrap_collection(context, refs, hints=hints)
-
-    @controller.protected()
-    def get_project(self, context, project_id):
-        ref = self.assignment_api.get_project(project_id)
-        return ProjectV3.wrap_member(context, ref)
-
-    @controller.protected()
-    @validation.validated(schema.project_update, 'project')
-    def update_project(self, context, project_id, project):
-        self._require_matching_id(project_id, project)
-        self._require_matching_domain_id(
-            project_id, project, self.assignment_api.get_project)
-        ref = self.assignment_api.update_project(project_id, project)
-        return ProjectV3.wrap_member(context, ref)
-
-    @controller.protected()
-    def delete_project(self, context, project_id):
-        return self.assignment_api.delete_project(project_id)
+        return ProjectAssignmentV3.wrap_collection(context, refs, hints=hints)
 
 
-@dependency.requires('assignment_api', 'identity_api')
+@dependency.requires('role_api')
 class RoleV3(controller.V3Controller):
+    """The V3 Role CRUD APIs."""
+
     collection_name = 'roles'
     member_name = 'role'
 
     def __init__(self):
         super(RoleV3, self).__init__()
-        self.get_member_from_driver = self.assignment_api.get_role
+        self.get_member_from_driver = self.role_api.get_role
 
     @controller.protected()
     @validation.validated(schema.role_create, 'role')
     def create_role(self, context, role):
-        ref = self._assign_unique_id(self._normalize_dict(role))
-        ref = self.assignment_api.create_role(ref['id'], ref)
+        if role['name'] == CONF.member_role_name:
+            # Use the configured member role ID when creating the configured
+            # member role name. This avoids the potential of creating a
+            # "member" role with an unexpected ID.
+            role['id'] = CONF.member_role_id
+        else:
+            role = self._assign_unique_id(role)
+
+        ref = self._normalize_dict(role)
+
+        initiator = notifications._get_request_audit_info(context)
+        ref = self.role_api.create_role(ref['id'], ref, initiator)
         return RoleV3.wrap_member(context, ref)
 
     @controller.filterprotected('name')
     def list_roles(self, context, filters):
         hints = RoleV3.build_driver_hints(context, filters)
-        refs = self.assignment_api.list_roles(
+        refs = self.role_api.list_roles(
             hints=hints)
         return RoleV3.wrap_collection(context, refs, hints=hints)
 
     @controller.protected()
     def get_role(self, context, role_id):
-        ref = self.assignment_api.get_role(role_id)
+        ref = self.role_api.get_role(role_id)
         return RoleV3.wrap_member(context, ref)
 
     @controller.protected()
     @validation.validated(schema.role_update, 'role')
     def update_role(self, context, role_id, role):
         self._require_matching_id(role_id, role)
-
-        ref = self.assignment_api.update_role(role_id, role)
+        initiator = notifications._get_request_audit_info(context)
+        ref = self.role_api.update_role(role_id, role, initiator)
         return RoleV3.wrap_member(context, ref)
 
     @controller.protected()
     def delete_role(self, context, role_id):
-        self.assignment_api.delete_role(role_id)
+        initiator = notifications._get_request_audit_info(context)
+        self.role_api.delete_role(role_id, initiator)
+
+
+@dependency.requires('assignment_api', 'identity_api', 'resource_api',
+                     'role_api')
+class GrantAssignmentV3(controller.V3Controller):
+    """The V3 Grant Assignment APIs."""
+
+    collection_name = 'roles'
+    member_name = 'role'
+
+    def __init__(self):
+        super(GrantAssignmentV3, self).__init__()
+        self.get_member_from_driver = self.role_api.get_role
 
     def _require_domain_xor_project(self, domain_id, project_id):
-        if (domain_id and project_id) or (not domain_id and not project_id):
+        if domain_id and project_id:
             msg = _('Specify a domain or project, not both')
             raise exception.ValidationError(msg)
+        if not domain_id and not project_id:
+            msg = _('Specify one of domain or project')
+            raise exception.ValidationError(msg)
 
     def _require_user_xor_group(self, user_id, group_id):
-        if (user_id and group_id) or (not user_id and not group_id):
+        if user_id and group_id:
             msg = _('Specify a user or group, not both')
             raise exception.ValidationError(msg)
+        if not user_id and not group_id:
+            msg = _('Specify one of user or group')
+            raise exception.ValidationError(msg)
 
     def _check_if_inherited(self, context):
         return (CONF.os_inherit.enabled and
@@ -502,7 +375,7 @@ class RoleV3(controller.V3Controller):
         """
         ref = {}
         if role_id:
-            ref['role'] = self.assignment_api.get_role(role_id)
+            ref['role'] = self.role_api.get_role(role_id)
         if user_id:
             try:
                 ref['user'] = self.identity_api.get_user(user_id)
@@ -513,9 +386,9 @@ class RoleV3(controller.V3Controller):
             ref['group'] = self.identity_api.get_group(group_id)
 
         if domain_id:
-            ref['domain'] = self.assignment_api.get_domain(domain_id)
+            ref['domain'] = self.resource_api.get_domain(domain_id)
         else:
-            ref['project'] = self.assignment_api.get_project(project_id)
+            ref['project'] = self.resource_api.get_project(project_id)
 
         self.check_protection(context, protection, ref)
 
@@ -540,7 +413,7 @@ class RoleV3(controller.V3Controller):
         refs = self.assignment_api.list_grants(
             user_id, group_id, domain_id, project_id,
             self._check_if_inherited(context))
-        return RoleV3.wrap_collection(context, refs)
+        return GrantAssignmentV3.wrap_collection(context, refs)
 
     @controller.protected(callback=_check_grant_protection)
     def check_grant(self, context, role_id, user_id=None,
@@ -569,8 +442,9 @@ class RoleV3(controller.V3Controller):
             self._check_if_inherited(context), context)
 
 
-@dependency.requires('assignment_api', 'identity_api')
+@dependency.requires('assignment_api', 'identity_api', 'resource_api')
 class RoleAssignmentV3(controller.V3Controller):
+    """The V3 Role Assignment APIs, really just list_role_assignment()."""
 
     # TODO(henry-nash): The current implementation does not provide a full
     # first class entity for role-assignment. There is no role_assignment_id
@@ -642,7 +516,13 @@ class RoleAssignmentV3(controller.V3Controller):
         if 'project_id' in entity:
             formatted_entity['scope'] = (
                 {'project': {'id': entity['project_id']}})
-            target_link = '/projects/%s' % entity['project_id']
+            if 'inherited_to_projects' in entity:
+                formatted_entity['scope']['OS-INHERIT:inherited_to'] = (
+                    'projects')
+                target_link = '/OS-INHERIT/projects/%s' % entity['project_id']
+                suffix = '/inherited_to_projects'
+            else:
+                target_link = '/projects/%s' % entity['project_id']
         if 'domain_id' in entity:
             formatted_entity['scope'] = (
                 {'domain': {'id': entity['domain_id']}})
@@ -709,10 +589,10 @@ class RoleAssignmentV3(controller.V3Controller):
                 if 'role' in ref and 'id' in ref['role']:
                     role_id = ref['role']['id']
                 LOG.warning(
-                    _('Group %(group)s not found for role-assignment - '
-                      '%(target)s with Role: %(role)s'), {
-                          'group': ref['group']['id'], 'target': target,
-                          'role': role_id})
+                    _LW('Group %(group)s not found for role-assignment - '
+                        '%(target)s with Role: %(role)s'), {
+                            'group': ref['group']['id'], 'target': target,
+                            'role': role_id})
             return members
 
         def _build_user_assignment_equivalent_of_group(
@@ -733,8 +613,8 @@ class RoleAssignmentV3(controller.V3Controller):
                               (group_id, user['id'])))
             return user_entry
 
-        def _build_project_equivalent_of_user_domain_role(
-                project_id, domain_id, template):
+        def _build_project_equivalent_of_user_target_role(
+                project_id, target_id, target_type, template):
             """Create a user project assignment equivalent to the domain one.
 
             The template has had the 'domain' entity removed, so
@@ -747,14 +627,15 @@ class RoleAssignmentV3(controller.V3Controller):
             project_entry['links']['assignment'] = (
                 self.base_url(
                     context,
-                    '/OS-INHERIT/domains/%s/users/%s/roles/%s'
+                    '/OS-INHERIT/%s/%s/users/%s/roles/%s'
                     '/inherited_to_projects' % (
-                        domain_id, project_entry['user']['id'],
+                        target_type, target_id, project_entry['user']['id'],
                         project_entry['role']['id'])))
             return project_entry
 
-        def _build_project_equivalent_of_group_domain_role(
-                user_id, group_id, project_id, domain_id, template):
+        def _build_project_equivalent_of_group_target_role(
+                user_id, group_id, project_id,
+                target_id, target_type, template):
             """Create a user project equivalent to the domain group one.
 
             The template has had the 'domain' and 'group' entities removed, so
@@ -767,9 +648,9 @@ class RoleAssignmentV3(controller.V3Controller):
             project_entry['scope']['project'] = {'id': project_id}
             project_entry['links']['assignment'] = (
                 self.base_url(context,
-                              '/OS-INHERIT/domains/%s/groups/%s/roles/%s'
+                              '/OS-INHERIT/%s/%s/groups/%s/roles/%s'
                               '/inherited_to_projects' % (
-                                  domain_id, group_id,
+                                  target_type, target_id, group_id,
                                   project_entry['role']['id'])))
             project_entry['links']['membership'] = (
                 self.base_url(context, '/groups/%s/users/%s' %
@@ -793,16 +674,30 @@ class RoleAssignmentV3(controller.V3Controller):
         new_refs = []
         for r in refs:
             if 'OS-INHERIT:inherited_to' in r['scope']:
-                # It's an inherited domain role - so get the list of projects
-                # owned by this domain. A domain scope is guaranteed since we
-                # checked this when we built the refs list
-                project_ids = (
-                    [x['id'] for x in
-                        self.assignment_api.list_projects_in_domain(
-                            r['scope']['domain']['id'])])
-                base_entry = copy.deepcopy(r)
-                domain_id = base_entry['scope']['domain']['id']
-                base_entry['scope'].pop('domain')
+                if 'domain' in r['scope']:
+                    # It's an inherited domain role - so get the list of
+                    # projects owned by this domain.
+                    project_ids = (
+                        [x['id'] for x in
+                            self.resource_api.list_projects_in_domain(
+                                r['scope']['domain']['id'])])
+                    base_entry = copy.deepcopy(r)
+                    target_type = 'domains'
+                    target_id = base_entry['scope']['domain']['id']
+                    base_entry['scope'].pop('domain')
+                else:
+                    # It's an inherited project role - so get the list of
+                    # projects in this project subtree.
+                    project_id = r['scope']['project']['id']
+                    project_ids = (
+                        [x['id'] for x in
+                            self.resource_api.list_projects_in_subtree(
+                                project_id)])
+                    base_entry = copy.deepcopy(r)
+                    target_type = 'projects'
+                    target_id = base_entry['scope']['project']['id']
+                    base_entry['scope'].pop('project')
+
                 # For each project, create an equivalent role assignment
                 for p in project_ids:
                     # If it's a group assignment, then create equivalent user
@@ -814,14 +709,14 @@ class RoleAssignmentV3(controller.V3Controller):
                         sub_entry.pop('group')
                         for m in members:
                             new_entry = (
-                                _build_project_equivalent_of_group_domain_role(
+                                _build_project_equivalent_of_group_target_role(
                                     m['id'], group_id, p,
-                                    domain_id, sub_entry))
+                                    target_id, target_type, sub_entry))
                             new_refs.append(new_entry)
                     else:
                         new_entry = (
-                            _build_project_equivalent_of_user_domain_role(
-                                p, domain_id, base_entry))
+                            _build_project_equivalent_of_user_target_role(
+                                p, target_id, target_type, base_entry))
                         new_refs.append(new_entry)
             elif 'group' in r:
                 # It's a non-inherited group role assignment, so get the list
@@ -842,33 +737,43 @@ class RoleAssignmentV3(controller.V3Controller):
 
         return new_refs
 
-    def _query_filter_is_true(self, filter_value):
-        """Determine if bool query param is 'True'.
-
-        We treat this the same way as we do for policy
-        enforcement:
-
-        {bool_param}=0 is treated as False
-
-        Any other value is considered to be equivalent to
-        True, including the absence of a value
-
-        """
-
-        if (isinstance(filter_value, six.string_types) and
-                filter_value == '0'):
-            val = False
-        else:
-            val = True
-        return val
-
     def _filter_inherited(self, entry):
         if ('inherited_to_projects' in entry and
                 not CONF.os_inherit.enabled):
-                    return False
+            return False
         else:
             return True
 
+    def _assert_effective_filters(self, inherited, group, domain):
+        """Assert that useless filter combinations are avoided.
+
+        In effective mode, the following filter combinations are useless, since
+        they would always return an empty list of role assignments:
+        - group id, since no group assignment is returned in effective mode;
+        - domain id and inherited, since no domain inherited assignment is
+        returned in effective mode.
+
+        """
+        if group:
+            msg = _('Combining effective and group filter will always '
+                    'result in an empty list.')
+            raise exception.ValidationError(msg)
+
+        if inherited and domain:
+            msg = _('Combining effective, domain and inherited filters will '
+                    'always result in an empty list.')
+            raise exception.ValidationError(msg)
+
+    def _assert_domain_nand_project(self, domain_id, project_id):
+        if domain_id and project_id:
+            msg = _('Specify a domain or project, not both')
+            raise exception.ValidationError(msg)
+
+    def _assert_user_nand_group(self, user_id, group_id):
+        if user_id and group_id:
+            msg = _('Specify a user or group, not both')
+            raise exception.ValidationError(msg)
+
     @controller.filterprotected('group.id', 'role.id',
                                 'scope.domain.id', 'scope.project.id',
                                 'scope.OS-INHERIT:inherited_to', 'user.id')
@@ -880,16 +785,35 @@ class RoleAssignmentV3(controller.V3Controller):
         # to pass the filters into the driver call, so that the list size is
         # kept a minimum.
 
+        params = context['query_string']
+        effective = 'effective' in params and (
+            self.query_filter_is_true(params['effective']))
+
+        if 'scope.OS-INHERIT:inherited_to' in params:
+            inherited = (
+                params['scope.OS-INHERIT:inherited_to'] == 'projects')
+        else:
+            # None means querying both inherited and direct assignments
+            inherited = None
+
+        self._assert_domain_nand_project(params.get('scope.domain.id'),
+                                         params.get('scope.project.id'))
+        self._assert_user_nand_group(params.get('user.id'),
+                                     params.get('group.id'))
+
+        if effective:
+            self._assert_effective_filters(inherited=inherited,
+                                           group=params.get('group.id'),
+                                           domain=params.get(
+                                               'scope.domain.id'))
+
         hints = self.build_driver_hints(context, filters)
         refs = self.assignment_api.list_role_assignments()
         formatted_refs = (
             [self._format_entity(context, x) for x in refs
              if self._filter_inherited(x)])
 
-        if ('effective' in context['query_string'] and
-                self._query_filter_is_true(
-                    context['query_string']['effective'])):
-
+        if effective:
             formatted_refs = self._expand_indirect_assignments(context,
                                                                formatted_refs)
 
