@@ -12,13 +12,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from oslo.config import cfg
-from oslo import messaging
+from oslo_config import cfg
+import oslo_messaging
 
 
-_DEFAULT_AUTH_METHODS = ['external', 'password', 'token']
+_DEFAULT_AUTH_METHODS = ['external', 'password', 'token', 'oauth1']
 _CERTFILE = '/etc/keystone/ssl/certs/signing_cert.pem'
 _KEYFILE = '/etc/keystone/ssl/private/signing_key.pem'
+_SSO_CALLBACK = '/etc/keystone/sso_callback_template.html'
 
 
 FILE_OPTIONS = {
@@ -31,18 +32,6 @@ FILE_OPTIONS = {
                         'AdminTokenAuthMiddleware from your paste '
                         'application pipelines (for example, in '
                         'keystone-paste.ini).'),
-        cfg.StrOpt('public_bind_host',
-                   default='0.0.0.0',
-                   deprecated_opts=[cfg.DeprecatedOpt('bind_host',
-                                                      group='DEFAULT')],
-                   help='The IP address of the network interface for the '
-                        'public service to listen on.'),
-        cfg.StrOpt('admin_bind_host',
-                   default='0.0.0.0',
-                   deprecated_opts=[cfg.DeprecatedOpt('bind_host',
-                                                      group='DEFAULT')],
-                   help='The IP address of the network interface for the '
-                        'admin service to listen on.'),
         cfg.IntOpt('compute_port', default=8774,
                    help='(Deprecated) The port which the OpenStack Compute '
                         'service listens on. This option was only used for '
@@ -51,44 +40,30 @@ FILE_OPTIONS = {
                         '"$(compute_port)s" substitution with the static port '
                         'of the compute service. As of Juno, this option is '
                         'deprecated and will be removed in the L release.'),
-        cfg.IntOpt('admin_port', default=35357,
-                   help='The port number which the admin service listens '
-                        'on.'),
-        cfg.IntOpt('public_port', default=5000,
-                   help='The port number which the public service listens '
-                        'on.'),
         cfg.StrOpt('public_endpoint',
                    help='The base public endpoint URL for Keystone that is '
                         'advertised to clients (NOTE: this does NOT affect '
                         'how Keystone listens for connections). '
                         'Defaults to the base host URL of the request. E.g. a '
-                        'request to http://server:5000/v2.0/users will '
+                        'request to http://server:5000/v3/users will '
                         'default to http://server:5000. You should only need '
                         'to set this value if the base URL contains a path '
-                        '(e.g. /prefix/v2.0) or the endpoint should be found '
+                        '(e.g. /prefix/v3) or the endpoint should be found '
                         'on a different server.'),
         cfg.StrOpt('admin_endpoint',
                    help='The base admin endpoint URL for Keystone that is '
                         'advertised to clients (NOTE: this does NOT affect '
                         'how Keystone listens for connections). '
                         'Defaults to the base host URL of the request. E.g. a '
-                        'request to http://server:35357/v2.0/users will '
+                        'request to http://server:35357/v3/users will '
                         'default to http://server:35357. You should only need '
                         'to set this value if the base URL contains a path '
-                        '(e.g. /prefix/v2.0) or the endpoint should be found '
+                        '(e.g. /prefix/v3) or the endpoint should be found '
                         'on a different server.'),
-        cfg.IntOpt('public_workers',
-                   help='The number of worker processes to serve the public '
-                        'WSGI application. Defaults to number of CPUs '
-                        '(minimum of 2).'),
-        cfg.IntOpt('admin_workers',
-                   help='The number of worker processes to serve the admin '
-                        'WSGI application. Defaults to number of CPUs '
-                        '(minimum of 2).'),
-        # default max request size is 112k
-        cfg.IntOpt('max_request_body_size', default=114688,
-                   help='Enforced by optional sizelimit middleware '
-                        '(keystone.middleware:RequestBodySizeLimiter).'),
+        cfg.IntOpt('max_project_tree_depth', default=5,
+                   help='Maximum depth of the project hierarchy. WARNING: '
+                        'setting it to a large value may adversely impact '
+                        'performance.'),
         cfg.IntOpt('max_param_size', default=64,
                    help='Limit the sizes of user & project ID/names.'),
         # we allow tokens to be a bit larger to accommodate PKI
@@ -97,38 +72,18 @@ FILE_OPTIONS = {
                         'exception for token values.'),
         cfg.StrOpt('member_role_id',
                    default='9fe2ff9ee4384b1894a90878d3e92bab',
-                   help='During a SQL upgrade member_role_id will be used '
-                        'to create a new role that will replace records in '
-                        'the assignment table with explicit role grants. '
-                        'After migration, the member_role_id will be used in '
-                        'the API add_user_to_project.'),
+                   help='Similar to the member_role_name option, this '
+                        'represents the default role ID used to associate '
+                        'users with their default projects in the v2 API. '
+                        'This will be used as the explicit role where one is '
+                        'not specified by the v2 API.'),
         cfg.StrOpt('member_role_name', default='_member_',
-                   help='During a SQL upgrade member_role_name will be used '
-                        'to create a new role that will replace records in '
-                        'the assignment table with explicit role grants. '
-                        'After migration, member_role_name will be ignored.'),
+                   help='This is the role name used in combination with the '
+                        'member_role_id option; see that option for more '
+                        'detail.'),
         cfg.IntOpt('crypt_strength', default=40000,
                    help='The value passed as the keyword "rounds" to '
                         'passlib\'s encrypt method.'),
-        cfg.BoolOpt('wsgi_keep_alive', default=True,
-                    help="If set to false, disables keepalives on the server; "
-                         "all connections will be closed after serving one "
-                         "request."),
-        cfg.IntOpt('client_socket_timeout', default=900,
-                   help="Timeout for socket operations on a client "
-                        "connection. If an incoming connection is idle for "
-                        "this number of seconds it will be closed. A value "
-                        "of '0' means wait forever."),
-        cfg.BoolOpt('tcp_keepalive', default=False,
-                    help='Set this to true if you want to enable '
-                         'TCP_KEEPALIVE on server sockets, i.e. sockets used '
-                         'by the Keystone wsgi server for client '
-                         'connections.'),
-        cfg.IntOpt('tcp_keepidle',
-                   default=600,
-                   help='Sets the value of TCP_KEEPIDLE in seconds for each '
-                        'server socket. Only applies if tcp_keepalive is '
-                        'true. Not supported on OS X.'),
         cfg.IntOpt('list_limit',
                    help='The maximum number of entities that will be '
                         'returned in a collection, with no limit set by '
@@ -149,7 +104,13 @@ FILE_OPTIONS = {
                          'exceeds the maximum length, the operation will fail '
                          'with an HTTP 403 Forbidden error. If set to false, '
                          'passwords are automatically truncated to the '
-                         'maximum length.')],
+                         'maximum length.'),
+        cfg.StrOpt('secure_proxy_ssl_header',
+                   help='The HTTP header used to determine the scheme for the '
+                        'original request, even if it was removed by an SSL '
+                        'terminating proxy. Typical value is '
+                        '"HTTP_X_FORWARDED_PROTO".'),
+    ],
     'identity': [
         cfg.StrOpt('default_domain_id', default='default',
                    help='This references the domain to use for all '
@@ -166,11 +127,22 @@ FILE_OPTIONS = {
                     default=False,
                     help='A subset (or all) of domains can have their own '
                          'identity driver, each with their own partial '
-                         'configuration file in a domain configuration '
-                         'directory. Only values specific to the domain '
-                         'need to be placed in the domain specific '
-                         'configuration file. This feature is disabled by '
+                         'configuration options, stored in either the '
+                         'resource backend or in a file in a domain '
+                         'configuration directory (depending on the setting '
+                         'of domain_configurations_from_database). Only '
+                         'values specific to the domain need to be specified '
+                         'in this manner. This feature is disabled by '
                          'default; set to true to enable.'),
+        cfg.BoolOpt('domain_configurations_from_database',
+                    default=False,
+                    help='Extract the domain specific configuration options '
+                         'from the resource backend where they have been '
+                         'stored with the domain data. This feature is '
+                         'disabled by default (in which case the domain '
+                         'specific options will be loaded from files in the '
+                         'domain configuration directory); set to true to '
+                         'enable.'),
         cfg.StrOpt('domain_config_dir',
                    default='/etc/keystone/domains',
                    help='Path for Keystone to locate the domain specific '
@@ -180,6 +152,13 @@ FILE_OPTIONS = {
                    default=('keystone.identity.backends'
                             '.sql.Identity'),
                    help='Identity backend driver.'),
+        cfg.BoolOpt('caching', default=True,
+                    help='Toggle for identity caching. This has no '
+                         'effect unless global caching is enabled.'),
+        cfg.IntOpt('cache_time', default=600,
+                   help='Time to cache identity data (in seconds). This has '
+                        'no effect unless global and identity caching are '
+                        'enabled.'),
         cfg.IntOpt('max_password_length', default=4096,
                    help='Maximum supported length for user passwords; '
                         'decrease to improve performance.'),
@@ -225,13 +204,32 @@ FILE_OPTIONS = {
         cfg.BoolOpt('enabled', default=True,
                     help='Delegation and impersonation features can be '
                          'optionally disabled.'),
+        cfg.BoolOpt('allow_redelegation', default=False,
+                    help='Enable redelegation feature.'),
+        cfg.IntOpt('max_redelegation_count', default=3,
+                   help='Maximum depth of trust redelegation.'),
         cfg.StrOpt('driver',
                    default='keystone.trust.backends.sql.Trust',
                    help='Trust backend driver.')],
     'os_inherit': [
         cfg.BoolOpt('enabled', default=False,
                     help='role-assignment inheritance to projects from '
-                         'owning domain can be optionally enabled.'),
+                         'owning domain or from projects higher in the '
+                         'hierarchy can be optionally enabled.'),
+    ],
+    'fernet_tokens': [
+        cfg.StrOpt('key_repository',
+                   default='/etc/keystone/fernet-keys/',
+                   help='Directory containing Fernet token keys.'),
+        cfg.IntOpt('max_active_keys',
+                   default=3,
+                   help='This controls how many keys are held in rotation by '
+                        'keystone-manage fernet_rotate before they are '
+                        'discarded. The default value of 3 means that '
+                        'keystone will maintain one staged key, one primary '
+                        'key, and one secondary key. Increasing this value '
+                        'means that additional secondary keys will be kept in '
+                        'the rotation.'),
     ],
     'token': [
         cfg.ListOpt('bind', default=[],
@@ -247,21 +245,17 @@ FILE_OPTIONS = {
                    help='Amount of time a token should remain valid '
                         '(in seconds).'),
         cfg.StrOpt('provider',
+                   default='keystone.token.providers.uuid.Provider',
                    help='Controls the token construction, validation, and '
                         'revocation operations. Core providers are '
-                        '"keystone.token.providers.[pkiz|pki|uuid].'
-                        'Provider". The default provider is uuid.'),
+                        '"keystone.token.providers.[fernet|pkiz|pki|uuid].'
+                        'Provider".'),
         cfg.StrOpt('driver',
                    default='keystone.token.persistence.backends.sql.Token',
                    help='Token persistence backend driver.'),
         cfg.BoolOpt('caching', default=True,
                     help='Toggle for token system caching. This has no '
                          'effect unless global caching is enabled.'),
-        cfg.IntOpt('revocation_cache_time', default=3600,
-                   help='Time to cache the revocation list and the revocation '
-                        'events if revoke extension is enabled (in seconds). '
-                        'This has no effect unless global and token '
-                        'caching are enabled.'),
         cfg.IntOpt('cache_time',
                    help='Time to cache tokens (in seconds). This has no '
                         'effect unless global and token caching are '
@@ -274,6 +268,10 @@ FILE_OPTIONS = {
                     'list of tokens to revoke. Only disable if you are '
                     'switching to using the Revoke extension with a '
                     'backend other than KVS, which stores events in memory.'),
+        cfg.BoolOpt('allow_rescope_scoped_token', default=True,
+                    help='Allow rescoping of scoped token. Setting '
+                    'allow_rescoped_scoped_token to false prevents a user '
+                    'from exchanging a scoped token for any other token.'),
         cfg.StrOpt('hash_algorithm', default='md5',
                    help="The hash algorithm to use for PKI tokens. This can "
                         "be set to any algorithm that hashlib supports. "
@@ -284,7 +282,7 @@ FILE_OPTIONS = {
     ],
     'revoke': [
         cfg.StrOpt('driver',
-                   default='keystone.contrib.revoke.backends.kvs.Revoke',
+                   default='keystone.contrib.revoke.backends.sql.Revoke',
                    help='An implementation of the backend for persisting '
                         'revocation events.'),
         cfg.IntOpt('expiration_buffer', default=1800,
@@ -294,6 +292,12 @@ FILE_OPTIONS = {
         cfg.BoolOpt('caching', default=True,
                     help='Toggle for revocation event caching. This has no '
                          'effect unless global caching is enabled.'),
+        cfg.IntOpt('cache_time', default=3600,
+                   help='Time to cache the revocation list and the revocation '
+                        'events (in seconds). This has no effect unless '
+                        'global and token caching are enabled.',
+                   deprecated_opts=[cfg.DeprecatedOpt(
+                       'revocation_cache_time', group='token')]),
     ],
     'cache': [
         cfg.StrOpt('config_prefix', default='cache.keystone',
@@ -345,53 +349,36 @@ FILE_OPTIONS = {
         cfg.ListOpt('memcache_servers', default=['localhost:11211'],
                     help='Memcache servers in the format of "host:port".'
                     ' (dogpile.cache.memcache and keystone.cache.memcache_pool'
-                    ' backends only)'),
+                    ' backends only).'),
         cfg.IntOpt('memcache_dead_retry',
                    default=5 * 60,
                    help='Number of seconds memcached server is considered dead'
                    ' before it is tried again. (dogpile.cache.memcache and'
-                   ' keystone.cache.memcache_pool backends only)'),
+                   ' keystone.cache.memcache_pool backends only).'),
         cfg.IntOpt('memcache_socket_timeout',
                    default=3,
                    help='Timeout in seconds for every call to a server.'
                    ' (dogpile.cache.memcache and keystone.cache.memcache_pool'
-                   ' backends only)'),
+                   ' backends only).'),
         cfg.IntOpt('memcache_pool_maxsize',
                    default=10,
                    help='Max total number of open connections to every'
                    ' memcached server. (keystone.cache.memcache_pool backend'
-                   ' only)'),
+                   ' only).'),
         cfg.IntOpt('memcache_pool_unused_timeout',
                    default=60,
                    help='Number of seconds a connection to memcached is held'
                    ' unused in the pool before it is closed.'
-                   ' (keystone.cache.memcache_pool backend only)'),
+                   ' (keystone.cache.memcache_pool backend only).'),
         cfg.IntOpt('memcache_pool_connection_get_timeout',
                    default=10,
                    help='Number of seconds that an operation will wait to get '
                         'a memcache client connection.'),
     ],
     'ssl': [
-        cfg.BoolOpt('enable', default=False,
-                    help='Toggle for SSL support on the Keystone '
-                         'eventlet servers.'),
-        cfg.StrOpt('certfile',
-                   default="/etc/keystone/ssl/certs/keystone.pem",
-                   help='Path of the certfile for SSL. For non-production '
-                        'environments, you may be interested in using '
-                        '`keystone-manage ssl_setup` to generate self-signed '
-                        'certificates.'),
-        cfg.StrOpt('keyfile',
-                   default='/etc/keystone/ssl/private/keystonekey.pem',
-                   help='Path of the keyfile for SSL.'),
-        cfg.StrOpt('ca_certs',
-                   default='/etc/keystone/ssl/certs/ca.pem',
-                   help='Path of the ca cert file for SSL.'),
         cfg.StrOpt('ca_key',
                    default='/etc/keystone/ssl/private/cakey.pem',
                    help='Path of the CA key file for SSL.'),
-        cfg.BoolOpt('cert_required', default=False,
-                    help='Require client certificate.'),
         cfg.IntOpt('key_size', default=1024,
                    help='SSL key length (in bits) (auto generated '
                         'certificate).'),
@@ -404,9 +391,6 @@ FILE_OPTIONS = {
                         'certificate).'),
     ],
     'signing': [
-        cfg.StrOpt('token_format',
-                   help='Deprecated in favor of provider in the '
-                        '[token] section.'),
         cfg.StrOpt('certfile',
                    default=_CERTFILE,
                    help='Path of the certfile for token signing. For '
@@ -440,15 +424,56 @@ FILE_OPTIONS = {
         # the backend
         cfg.StrOpt('driver',
                    help='Assignment backend driver.'),
+    ],
+    'resource': [
+        cfg.StrOpt('driver',
+                   help='Resource backend driver. If a resource driver is '
+                        'not specified, the assignment driver will choose '
+                        'the resource driver.'),
         cfg.BoolOpt('caching', default=True,
-                    help='Toggle for assignment caching. This has no effect '
+                    deprecated_opts=[cfg.DeprecatedOpt('caching',
+                                                       group='assignment')],
+                    help='Toggle for resource caching. This has no effect '
                          'unless global caching is enabled.'),
         cfg.IntOpt('cache_time',
-                   help='TTL (in seconds) to cache assignment data. This has '
+                   deprecated_opts=[cfg.DeprecatedOpt('cache_time',
+                                                      group='assignment')],
+                   help='TTL (in seconds) to cache resource data. This has '
+                        'no effect unless global caching is enabled.'),
+        cfg.IntOpt('list_limit',
+                   deprecated_opts=[cfg.DeprecatedOpt('list_limit',
+                                                      group='assignment')],
+                   help='Maximum number of entities that will be returned '
+                        'in a resource collection.'),
+    ],
+    'domain_config': [
+        cfg.StrOpt('driver',
+                   default='keystone.resource.config_backends.sql.'
+                           'DomainConfig',
+                   help='Domain config backend driver.'),
+        cfg.BoolOpt('caching', default=True,
+                    help='Toggle for domain config caching. This has no '
+                         'effect unless global caching is enabled.'),
+        cfg.IntOpt('cache_time', default=300,
+                   help='TTL (in seconds) to cache domain config data. This '
+                        'has no effect unless domain config caching is '
+                        'enabled.'),
+    ],
+    'role': [
+        # The role driver has no default for backward compatibility reasons.
+        # If role driver is not specified, the assignment driver chooses
+        # the backend
+        cfg.StrOpt('driver',
+                   help='Role backend driver.'),
+        cfg.BoolOpt('caching', default=True,
+                    help='Toggle for role caching. This has no effect '
+                         'unless global caching is enabled.'),
+        cfg.IntOpt('cache_time',
+                   help='TTL (in seconds) to cache role data. This has '
                         'no effect unless global caching is enabled.'),
         cfg.IntOpt('list_limit',
                    help='Maximum number of entities that will be returned '
-                        'in an assignment collection.'),
+                        'in a role collection.'),
     ],
     'credential': [
         cfg.StrOpt('driver',
@@ -473,6 +498,31 @@ FILE_OPTIONS = {
         cfg.StrOpt('assertion_prefix', default='',
                    help='Value to be used when filtering assertion parameters '
                         'from the environment.'),
+        cfg.StrOpt('remote_id_attribute',
+                   help='Value to be used to obtain the entity ID of the '
+                        'Identity Provider from the environment (e.g. if '
+                        'using the mod_shib plugin this value is '
+                        '`Shib-Identity-Provider`).'),
+        cfg.StrOpt('federated_domain_name', default='Federated',
+                   help='A domain name that is reserved to allow federated '
+                        'ephemeral users to have a domain concept. Note that '
+                        'an admin will not be able to create a domain with '
+                        'this name or update an existing domain to this '
+                        'name. You are not advised to change this value '
+                        'unless you really have to. Changing this option '
+                        'to empty string or None will not have any impact and '
+                        'default name will be used.'),
+        cfg.MultiStrOpt('trusted_dashboard', default=[],
+                        help='A list of trusted dashboard hosts. Before '
+                             'accepting a Single Sign-On request to return a '
+                             'token, the origin host must be a member of the '
+                             'trusted_dashboard list. This configuration '
+                             'option may be repeated for multiple values. '
+                             'For example: trusted_dashboard=http://acme.com '
+                             'trusted_dashboard=http://beta.com'),
+        cfg.StrOpt('sso_callback_template', default=_SSO_CALLBACK,
+                   help='Location of Single Sign-On callback handler, will '
+                        'return a token to a trusted dashboard host.'),
     ],
     'policy': [
         cfg.StrOpt('driver',
@@ -482,11 +532,6 @@ FILE_OPTIONS = {
                    help='Maximum number of entities that will be returned '
                         'in a policy collection.'),
     ],
-    'ec2': [
-        cfg.StrOpt('driver',
-                   default='keystone.contrib.ec2.backends.kvs.Ec2',
-                   help='EC2Credential backend driver.'),
-    ],
     'endpoint_filter': [
         cfg.StrOpt('driver',
                    default='keystone.contrib.endpoint_filter.backends'
@@ -502,12 +547,6 @@ FILE_OPTIONS = {
                            '.sql.EndpointPolicy',
                    help='Endpoint policy backend driver'),
     ],
-    'stats': [
-        cfg.StrOpt('driver',
-                   default=('keystone.contrib.stats.backends'
-                            '.kvs.Stats'),
-                   help='Stats backend driver.'),
-    ],
     'ldap': [
         cfg.StrOpt('url', default='ldap://localhost',
                    help='URL for connecting to the LDAP server.'),
@@ -800,6 +839,9 @@ FILE_OPTIONS = {
         cfg.StrOpt('external',
                    default='keystone.auth.plugins.external.DefaultDomain',
                    help='The external (REMOTE_USER) auth plugin module.'),
+        cfg.StrOpt('oauth1',
+                   default='keystone.auth.plugins.oauth1.OAuth',
+                   help='The oAuth1.0 auth plugin module.'),
     ],
     'paste_deploy': [
         cfg.StrOpt('config_file', default='keystone-paste.ini',
@@ -857,17 +899,6 @@ FILE_OPTIONS = {
         cfg.IntOpt('list_limit',
                    help='Maximum number of entities that will be returned '
                         'in a catalog collection.'),
-        cfg.ListOpt('endpoint_substitution_whitelist',
-                    default=['tenant_id', 'user_id', 'public_bind_host',
-                             'admin_bind_host', 'compute_host', 'compute_port',
-                             'admin_port', 'public_port', 'public_endpoint',
-                             'admin_endpoint'],
-                    help='(Deprecated) List of possible substitutions for use '
-                         'in formatting endpoints. Use caution when modifying '
-                         'this list. It will give users with permission to '
-                         'create endpoints the ability to see those values '
-                         'in your configuration file. This option will be '
-                         'removed in Juno.'),
     ],
     'kvs': [
         cfg.ListOpt('backends', default=[],
@@ -884,7 +915,8 @@ FILE_OPTIONS = {
                          'for debugging purposes, it is highly recommended '
                          'to always leave this set to true.'),
         cfg.IntOpt('default_lock_timeout', default=5,
-                   help='Default lock timeout for distributed locking.'),
+                   help='Default lock timeout (in seconds) for distributed '
+                        'locking.'),
     ],
     'saml': [
         cfg.IntOpt('assertion_expiration_time', default=3600,
@@ -946,12 +978,106 @@ FILE_OPTIONS = {
                    help='Path to the Identity Provider Metadata file. '
                         'This file should be generated with the '
                         'keystone-manage saml_idp_metadata command.'),
+        cfg.StrOpt('relay_state_prefix',
+                   default='ss:mem:',
+                   help='The prefix to use for the RelayState SAML '
+                        'attribute, used when generating ECP wrapped '
+                        'assertions.'),
+    ],
+    'eventlet_server': [
+        cfg.IntOpt('public_workers',
+                   deprecated_name='public_workers',
+                   deprecated_group='DEFAULT',
+                   deprecated_for_removal=True,
+                   help='The number of worker processes to serve the public '
+                        'eventlet application. Defaults to number of CPUs '
+                        '(minimum of 2).'),
+        cfg.IntOpt('admin_workers',
+                   deprecated_name='admin_workers',
+                   deprecated_group='DEFAULT',
+                   deprecated_for_removal=True,
+                   help='The number of worker processes to serve the admin '
+                        'eventlet application. Defaults to number of CPUs '
+                        '(minimum of 2).'),
+        cfg.StrOpt('public_bind_host',
+                   default='0.0.0.0',
+                   deprecated_opts=[cfg.DeprecatedOpt('bind_host',
+                                                      group='DEFAULT'),
+                                    cfg.DeprecatedOpt('public_bind_host',
+                                                      group='DEFAULT'), ],
+                   deprecated_for_removal=True,
+                   help='The IP address of the network interface for the '
+                        'public service to listen on.'),
+        cfg.IntOpt('public_port', default=5000, deprecated_name='public_port',
+                   deprecated_group='DEFAULT',
+                   deprecated_for_removal=True,
+                   help='The port number which the public service listens '
+                        'on.'),
+        cfg.StrOpt('admin_bind_host',
+                   default='0.0.0.0',
+                   deprecated_opts=[cfg.DeprecatedOpt('bind_host',
+                                                      group='DEFAULT'),
+                                    cfg.DeprecatedOpt('admin_bind_host',
+                                                      group='DEFAULT')],
+                   deprecated_for_removal=True,
+                   help='The IP address of the network interface for the '
+                        'admin service to listen on.'),
+        cfg.IntOpt('admin_port', default=35357, deprecated_name='admin_port',
+                   deprecated_group='DEFAULT',
+                   deprecated_for_removal=True,
+                   help='The port number which the admin service listens '
+                        'on.'),
+        cfg.BoolOpt('tcp_keepalive', default=False,
+                    deprecated_name='tcp_keepalive',
+                    deprecated_group='DEFAULT',
+                    deprecated_for_removal=True,
+                    help='Set this to true if you want to enable '
+                         'TCP_KEEPALIVE on server sockets, i.e. sockets used '
+                         'by the Keystone wsgi server for client '
+                         'connections.'),
+        cfg.IntOpt('tcp_keepidle',
+                   default=600,
+                   deprecated_name='tcp_keepidle',
+                   deprecated_group='DEFAULT',
+                   deprecated_for_removal=True,
+                   help='Sets the value of TCP_KEEPIDLE in seconds for each '
+                        'server socket. Only applies if tcp_keepalive is '
+                        'true.'),
+    ],
+    'eventlet_server_ssl': [
+        cfg.BoolOpt('enable', default=False, deprecated_name='enable',
+                    deprecated_group='ssl',
+                    deprecated_for_removal=True,
+                    help='Toggle for SSL support on the Keystone '
+                         'eventlet servers.'),
+        cfg.StrOpt('certfile',
+                   default="/etc/keystone/ssl/certs/keystone.pem",
+                   deprecated_name='certfile', deprecated_group='ssl',
+                   deprecated_for_removal=True,
+                   help='Path of the certfile for SSL. For non-production '
+                        'environments, you may be interested in using '
+                        '`keystone-manage ssl_setup` to generate self-signed '
+                        'certificates.'),
+        cfg.StrOpt('keyfile',
+                   default='/etc/keystone/ssl/private/keystonekey.pem',
+                   deprecated_name='keyfile', deprecated_group='ssl',
+                   deprecated_for_removal=True,
+                   help='Path of the keyfile for SSL.'),
+        cfg.StrOpt('ca_certs',
+                   default='/etc/keystone/ssl/certs/ca.pem',
+                   deprecated_name='ca_certs', deprecated_group='ssl',
+                   deprecated_for_removal=True,
+                   help='Path of the CA cert file for SSL.'),
+        cfg.BoolOpt('cert_required', default=False,
+                    deprecated_name='cert_required', deprecated_group='ssl',
+                    deprecated_for_removal=True,
+                    help='Require client certificate.'),
     ],
 }
 
 
 CONF = cfg.CONF
-messaging.set_transport_defaults(control_exchange='keystone')
+oslo_messaging.set_transport_defaults(control_exchange='keystone')
 
 
 def _register_auth_plugin_opt(conf, option):
@@ -994,9 +1120,9 @@ def configure(conf=None):
 
 
 def list_opts():
-    """Return a list of oslo.config options available in Keystone.
+    """Return a list of oslo_config options available in Keystone.
 
-    The returned list includes all oslo.config options which are registered as
+    The returned list includes all oslo_config options which are registered as
     the "FILE_OPTIONS" in keystone.common.config. This list will not include
     the options from the oslo-incubator library or any options registered
     dynamically at run time.
@@ -1006,7 +1132,7 @@ def list_opts():
     second element will be registered. A group name of None corresponds to the
     [DEFAULT] group in config files.
 
-    This function is also discoverable via the 'oslo.config.opts' entry point
+    This function is also discoverable via the 'oslo_config.opts' entry point
     under the 'keystone.config.opts' namespace.
 
     The purpose of this is to allow tools like the Oslo sample config file
