@@ -12,21 +12,30 @@
 
 """Extensions supporting Federation."""
 
+import string
+
+from oslo_config import cfg
+from oslo_log import log
+import six
+from six.moves import urllib
+import webob
+
 from keystone.auth import controllers as auth_controllers
 from keystone.common import authorization
 from keystone.common import controller
 from keystone.common import dependency
 from keystone.common import validation
 from keystone.common import wsgi
-from keystone import config
 from keystone.contrib.federation import idp as keystone_idp
 from keystone.contrib.federation import schema
 from keystone.contrib.federation import utils
 from keystone import exception
+from keystone.i18n import _
 from keystone.models import token_model
 
 
-CONF = config.CONF
+CONF = cfg.CONF
+LOG = log.getLogger(__name__)
 
 
 class _ControllerBase(controller.V3Controller):
@@ -46,8 +55,10 @@ class IdentityProvider(_ControllerBase):
     collection_name = 'identity_providers'
     member_name = 'identity_provider'
 
-    _mutable_parameters = frozenset(['description', 'enabled'])
-    _public_parameters = frozenset(['id', 'enabled', 'description', 'links'])
+    _mutable_parameters = frozenset(['description', 'enabled', 'remote_ids'])
+    _public_parameters = frozenset(['id', 'enabled', 'description',
+                                    'remote_ids', 'links'
+                                    ])
 
     @classmethod
     def _add_related_links(cls, context, ref):
@@ -233,6 +244,7 @@ class MappingController(_ControllerBase):
         return MappingController.wrap_member(context, mapping_ref)
 
 
+@dependency.requires('federation_api')
 class Auth(auth_controllers.Auth):
 
     def federated_authentication(self, context, identity_provider, protocol):
@@ -254,18 +266,57 @@ class Auth(auth_controllers.Auth):
 
         return self.authenticate_for_token(context, auth=auth)
 
-    @validation.validated(schema.saml_create, 'auth')
-    def create_saml_assertion(self, context, auth):
-        """Exchange a scoped token for a SAML assertion.
+    def federated_sso_auth(self, context, protocol_id):
+        try:
+            remote_id_name = utils.get_remote_id_parameter(protocol_id)
+            remote_id = context['environment'][remote_id_name]
+        except KeyError:
+            msg = _('Missing entity ID from environment')
+            LOG.error(msg)
+            raise exception.Unauthorized(msg)
 
-        :param auth: Dictionary that contains a token id and region id
-        :returns: SAML Assertion based on properties from the token
-        """
+        if 'origin' in context['query_string']:
+            origin = context['query_string'].get('origin')
+            host = urllib.parse.unquote_plus(origin)
+        else:
+            msg = _('Request must have an origin query parameter')
+            LOG.error(msg)
+            raise exception.ValidationError(msg)
 
+        if host in CONF.federation.trusted_dashboard:
+            ref = self.federation_api.get_idp_from_remote_id(remote_id)
+            # NOTE(stevemar): the returned object is a simple dict that
+            # contains the idp_id and remote_id.
+            identity_provider = ref['idp_id']
+            res = self.federated_authentication(context, identity_provider,
+                                                protocol_id)
+            token_id = res.headers['X-Subject-Token']
+            return self.render_html_response(host, token_id)
+        else:
+            msg = _('%(host)s is not a trusted dashboard host')
+            msg = msg % {'host': host}
+            LOG.error(msg)
+            raise exception.Unauthorized(msg)
+
+    def render_html_response(self, host, token_id):
+        """Forms an HTML Form from a template with autosubmit."""
+
+        headers = [('Content-Type', 'text/html')]
+
+        with open(CONF.federation.sso_callback_template) as template:
+            src = string.Template(template.read())
+
+        subs = {'host': host, 'token': token_id}
+        body = src.substitute(subs)
+        return webob.Response(body=body, status='200',
+                              headerlist=headers)
+
+    def _create_base_saml_assertion(self, context, auth):
         issuer = CONF.saml.idp_entity_id
-        region_id = auth['scope']['region']['id']
-        region = self.catalog_api.get_region(region_id)
-        recipient = region['url']
+        sp_id = auth['scope']['service_provider']['id']
+        service_provider = self.federation_api.get_sp(sp_id)
+        utils.assert_enabled_service_provider_object(service_provider)
+        sp_url = service_provider.get('sp_url')
 
         token_id = auth['identity']['token']['id']
         token_data = self.token_provider_api.validate_token(token_id)
@@ -273,29 +324,68 @@ class Auth(auth_controllers.Auth):
         subject = token_ref.user_name
         roles = token_ref.role_names
 
-        if token_ref.project_scoped:
-            project = token_ref.project_name
-        else:
-            raise ValueError(_('Use a project scoped token when attempting to'
-                               'create a SAML assertion'))
+        if not token_ref.project_scoped:
+            action = _('Use a project scoped token when attempting to create '
+                       'a SAML assertion')
+            raise exception.ForbiddenAction(action=action)
 
+        project = token_ref.project_name
         generator = keystone_idp.SAMLGenerator()
-        response = generator.samlize_token(issuer, recipient, subject, roles,
+        response = generator.samlize_token(issuer, sp_url, subject, roles,
                                            project)
+        return (response, service_provider)
 
+    def _build_response_headers(self, service_provider):
+        return [('Content-Type', 'text/xml'),
+                ('X-sp-url', six.binary_type(service_provider['sp_url'])),
+                ('X-auth-url', six.binary_type(service_provider['auth_url']))]
+
+    @validation.validated(schema.saml_create, 'auth')
+    def create_saml_assertion(self, context, auth):
+        """Exchange a scoped token for a SAML assertion.
+
+        :param auth: Dictionary that contains a token and service provider ID
+        :returns: SAML Assertion based on properties from the token
+        """
+
+        t = self._create_base_saml_assertion(context, auth)
+        (response, service_provider) = t
+
+        headers = self._build_response_headers(service_provider)
         return wsgi.render_response(body=response.to_string(),
                                     status=('200', 'OK'),
-                                    headers=[('Content-Type', 'text/xml')])
+                                    headers=headers)
+
+    @validation.validated(schema.saml_create, 'auth')
+    def create_ecp_assertion(self, context, auth):
+        """Exchange a scoped token for an ECP assertion.
+
+        :param auth: Dictionary that contains a token and service provider ID
+        :returns: ECP Assertion based on properties from the token
+        """
+
+        t = self._create_base_saml_assertion(context, auth)
+        (saml_assertion, service_provider) = t
+        relay_state_prefix = service_provider.get('relay_state_prefix')
+
+        generator = keystone_idp.ECPGenerator()
+        ecp_assertion = generator.generate_ecp(saml_assertion,
+                                               relay_state_prefix)
+
+        headers = self._build_response_headers(service_provider)
+        return wsgi.render_response(body=ecp_assertion.to_string(),
+                                    status=('200', 'OK'),
+                                    headers=headers)
 
 
-@dependency.requires('assignment_api')
+@dependency.requires('assignment_api', 'resource_api')
 class DomainV3(controller.V3Controller):
     collection_name = 'domains'
     member_name = 'domain'
 
     def __init__(self):
         super(DomainV3, self).__init__()
-        self.get_member_from_driver = self.assignment_api.get_domain
+        self.get_member_from_driver = self.resource_api.get_domain
 
     @controller.protected()
     def list_domains_for_groups(self, context):
@@ -311,14 +401,14 @@ class DomainV3(controller.V3Controller):
         return DomainV3.wrap_collection(context, domains)
 
 
-@dependency.requires('assignment_api')
-class ProjectV3(controller.V3Controller):
+@dependency.requires('assignment_api', 'resource_api')
+class ProjectAssignmentV3(controller.V3Controller):
     collection_name = 'projects'
     member_name = 'project'
 
     def __init__(self):
-        super(ProjectV3, self).__init__()
-        self.get_member_from_driver = self.assignment_api.get_project
+        super(ProjectAssignmentV3, self).__init__()
+        self.get_member_from_driver = self.resource_api.get_project
 
     @controller.protected()
     def list_projects_for_groups(self, context):
@@ -331,7 +421,55 @@ class ProjectV3(controller.V3Controller):
         auth_context = context['environment'][authorization.AUTH_CONTEXT_ENV]
         projects = self.assignment_api.list_projects_for_groups(
             auth_context['group_ids'])
-        return ProjectV3.wrap_collection(context, projects)
+        return ProjectAssignmentV3.wrap_collection(context, projects)
+
+
+@dependency.requires('federation_api')
+class ServiceProvider(_ControllerBase):
+    """Service Provider representation."""
+
+    collection_name = 'service_providers'
+    member_name = 'service_provider'
+
+    _mutable_parameters = frozenset(['auth_url', 'description', 'enabled',
+                                     'relay_state_prefix', 'sp_url'])
+    _public_parameters = frozenset(['auth_url', 'id', 'enabled', 'description',
+                                    'links', 'relay_state_prefix', 'sp_url'])
+
+    @controller.protected()
+    @validation.validated(schema.service_provider_create, 'service_provider')
+    def create_service_provider(self, context, sp_id, service_provider):
+        service_provider = self._normalize_dict(service_provider)
+        service_provider.setdefault('enabled', False)
+        service_provider.setdefault('relay_state_prefix',
+                                    CONF.saml.relay_state_prefix)
+        ServiceProvider.check_immutable_params(service_provider)
+        sp_ref = self.federation_api.create_sp(sp_id, service_provider)
+        response = ServiceProvider.wrap_member(context, sp_ref)
+        return wsgi.render_response(body=response, status=('201', 'Created'))
+
+    @controller.protected()
+    def list_service_providers(self, context):
+        ref = self.federation_api.list_sps()
+        ref = [self.filter_params(x) for x in ref]
+        return ServiceProvider.wrap_collection(context, ref)
+
+    @controller.protected()
+    def get_service_provider(self, context, sp_id):
+        ref = self.federation_api.get_sp(sp_id)
+        return ServiceProvider.wrap_member(context, ref)
+
+    @controller.protected()
+    def delete_service_provider(self, context, sp_id):
+        self.federation_api.delete_sp(sp_id)
+
+    @controller.protected()
+    @validation.validated(schema.service_provider_update, 'service_provider')
+    def update_service_provider(self, context, sp_id, service_provider):
+        service_provider = self._normalize_dict(service_provider)
+        ServiceProvider.check_immutable_params(service_provider)
+        sp_ref = self.federation_api.update_sp(sp_id, service_provider)
+        return ServiceProvider.wrap_member(context, sp_ref)
 
 
 class SAMLMetadataV3(_ControllerBase):
