@@ -12,20 +12,19 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo_log import log
+from oslo_serialization import jsonutils
 import webob
 
 from keystone.common import extension
 from keystone.common import json_home
 from keystone.common import wsgi
 from keystone import exception
-from keystone.openstack.common import jsonutils
-from keystone.openstack.common import log
 
 
 LOG = log.getLogger(__name__)
 
 MEDIA_TYPE_JSON = 'application/vnd.openstack.identity-%s+json'
-MEDIA_TYPE_XML = 'application/vnd.openstack.identity-%s+xml'
 
 _VERSIONS = []
 
@@ -89,7 +88,7 @@ def register_version(version):
     _VERSIONS.append(version)
 
 
-class MimeTypes:
+class MimeTypes(object):
     JSON = 'application/json'
     JSON_HOME = 'application/json-home'
 
@@ -141,18 +140,15 @@ class Version(wsgi.Application):
                     {
                         'base': 'application/json',
                         'type': MEDIA_TYPE_JSON % 'v2.0'
-                    }, {
-                        'base': 'application/xml',
-                        'type': MEDIA_TYPE_XML % 'v2.0'
                     }
                 ]
             }
 
         if 'v3' in _VERSIONS:
             versions['v3'] = {
-                'id': 'v3.0',
+                'id': 'v3.4',
                 'status': 'stable',
-                'updated': '2013-03-06T00:00:00Z',
+                'updated': '2015-03-30T00:00:00Z',
                 'links': [
                     {
                         'rel': 'self',
@@ -163,9 +159,6 @@ class Version(wsgi.Application):
                     {
                         'base': 'application/json',
                         'type': MEDIA_TYPE_JSON % 'v3'
-                    }, {
-                        'base': 'application/xml',
-                        'type': MEDIA_TYPE_XML % 'v3'
                     }
                 ]
             }
