@@ -12,15 +12,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from oslo.utils import encodeutils
+from oslo_config import cfg
+from oslo_log import log
+from oslo_utils import encodeutils
 import six
 
-from keystone.common import config
-from keystone.i18n import _
-from keystone.openstack.common import log
+from keystone.i18n import _, _LW
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 # Tests use this to make exception message format errors fatal
@@ -46,7 +46,7 @@ class Error(Exception):
             if _FATAL_EXCEPTION_FORMAT_ERRORS:
                 raise
             else:
-                LOG.warning(_('missing exception kwargs (programmer error)'))
+                LOG.warning(_LW('missing exception kwargs (programmer error)'))
                 message = self.message_format
 
         super(Error, self).__init__(message)
@@ -62,8 +62,8 @@ class Error(Exception):
                 message = self.message_format % kwargs
             except UnicodeDecodeError:
                 try:
-                    kwargs = dict([(k, encodeutils.safe_decode(v)) for k, v in
-                                   six.iteritems(kwargs)])
+                    kwargs = {k: encodeutils.safe_decode(v)
+                              for k, v in six.iteritems(kwargs)}
                 except UnicodeDecodeError:
                     # NOTE(jamielennox): This is the complete failure case
                     # at least by showing the template we have some idea
@@ -115,6 +115,13 @@ class ValidationSizeError(Error):
     title = 'Bad Request'
 
 
+class CircularRegionHierarchyError(Error):
+    message_format = _("The specified parent region %(parent_region_id)s "
+                       "would create a circular region hierarchy.")
+    code = 400
+    title = 'Bad Request'
+
+
 class PasswordVerificationError(Error):
     message_format = _("The password length must be less than or equal "
                        "to %(size)i. The server could not comply with the "
@@ -217,6 +224,10 @@ class InvalidPolicyAssociation(Forbidden):
                        "Service: %(service_id)s, Region: %(region_id)s")
 
 
+class InvalidDomainConfig(Forbidden):
+    message_format = _("Invalid domain specific configuration: %(reason)s")
+
+
 class NotFound(Error):
     message_format = _("Could not find: %(target)s")
     code = 404
@@ -247,6 +258,12 @@ class RoleNotFound(NotFound):
     message_format = _("Could not find role: %(role_id)s")
 
 
+class RoleAssignmentNotFound(NotFound):
+    message_format = _("Could not find role assignment with role: "
+                       "%(role_id)s, user or group: %(actor_id)s, "
+                       "project or domain: %(target_id)s")
+
+
 class RegionNotFound(NotFound):
     message_format = _("Could not find region: %(region_id)s")
 
@@ -263,6 +280,10 @@ class ProjectNotFound(NotFound):
     message_format = _("Could not find project: %(project_id)s")
 
 
+class InvalidParentProject(NotFound):
+    message_format = _("Cannot create project with parent: %(project_id)s")
+
+
 class TokenNotFound(NotFound):
     message_format = _("Could not find token: %(token_id)s")
 
@@ -303,6 +324,10 @@ class IdentityProviderNotFound(NotFound):
     message_format = _("Could not find Identity Provider: %(idp_id)s")
 
 
+class ServiceProviderNotFound(NotFound):
+    message_format = _("Could not find Service Provider: %(sp_id)s")
+
+
 class FederatedProtocolNotFound(NotFound):
     message_format = _("Could not find federated protocol %(protocol_id)s for"
                        " Identity Provider: %(idp_id)s")
@@ -314,6 +339,11 @@ class PublicIDNotFound(NotFound):
     message_format = "%(id)s"
 
 
+class DomainConfigNotFound(NotFound):
+    message_format = _('Could not find %(group_or_option)s in domain '
+                       'configuration for domain %(domain_id)s')
+
+
 class Conflict(Error):
     message_format = _("Conflict occurred attempting to store %(type)s -"
                        " %(details)s")
@@ -321,12 +351,6 @@ class Conflict(Error):
     title = 'Conflict'
 
 
-class RequestTooLarge(Error):
-    message_format = _("Request is too large.")
-    code = 413
-    title = 'Request is too large.'
-
-
 class UnexpectedError(SecurityError):
     """Avoids exposing details of failures, unless in debug mode."""
     _message_format = _("An unexpected error prevented the server "
@@ -379,6 +403,13 @@ class MetadataFileError(UnexpectedError):
     message_format = _("Error while reading metadata file, %(reason)s")
 
 
+class AssignmentTypeCalculationError(UnexpectedError):
+    message_format = _(
+        'Unexpected combination of grant attributes - '
+        'User: %(user_id)s, Group: %(group_id)s, Project: %(project_id)s, '
+        'Domain: %(domain_id)s')
+
+
 class NotImplemented(Error):
     message_format = _("The action you have requested has not"
                        " been implemented.")
@@ -398,10 +429,15 @@ class ConfigFileNotFound(UnexpectedError):
                              "could not be found.")
 
 
+class KeysNotFound(UnexpectedError):
+    message_format = _('No encryption keys found; run keystone-manage '
+                       'fernet_setup to bootstrap one.')
+
+
 class MultipleSQLDriversInConfig(UnexpectedError):
-    message_format = _('The Keystone domain configuration file '
-                       '%(config_file)s defines an additional SQL driver - '
-                       'only one is permitted.')
+    message_format = _('The Keystone domain-specific configuration has '
+                       'specified more than one SQL driver (only one is '
+                       'permitted): %(source)s.')
 
 
 class MigrationNotProvided(Exception):
@@ -423,3 +459,11 @@ class SAMLSigningError(UnexpectedError):
                              'installed, or this is the result of '
                              'misconfiguration. Reason %(reason)s')
     title = 'Error signing SAML assertion'
+
+
+class OAuthHeadersMissingError(UnexpectedError):
+    debug_message_format = _('No Authorization headers found, cannot proceed '
+                             'with OAuth related calls, if running under '
+                             'HTTPd or Apache, ensure WSGIPassAuthorization '
+                             'is set to On.')
+    title = 'Error retrieving OAuth headers'
