@@ -12,22 +12,20 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo_config import cfg
+from oslo_log import log
+from oslo_middleware import sizelimit
+from oslo_serialization import jsonutils
 import six
-import webob.dec
 
 from keystone.common import authorization
-from keystone.common import config
-from keystone.common import serializer
-from keystone.common import utils
 from keystone.common import wsgi
 from keystone import exception
-from keystone.i18n import _
+from keystone.i18n import _LW
 from keystone.models import token_model
-from keystone.openstack.common import jsonutils
-from keystone.openstack.common import log
 from keystone.openstack.common import versionutils
 
-CONF = config.CONF
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -100,10 +98,6 @@ class JsonBodyMiddleware(wsgi.Middleware):
     Accepting arguments as JSON is useful for accepting data that may be more
     complex than simple primitives.
 
-    In this case we accept it as urlencoded data under the key 'json' as in
-    json=<urlencoded_json> but this could be extended to accept raw JSON
-    in the POST body.
-
     Filters out the parameters `self`, `context` and anything beginning with
     an underscore.
 
@@ -149,59 +143,32 @@ class JsonBodyMiddleware(wsgi.Middleware):
 
 
 class XmlBodyMiddleware(wsgi.Middleware):
-    """De/serializes XML to/from JSON."""
+    """De/serialize XML to/from JSON."""
+
+    def print_warning(self):
+        LOG.warning(_LW('XML support has been removed as of the Kilo release '
+                        'and should not be referenced or used in deployment. '
+                        'Please remove references to XmlBodyMiddleware from '
+                        'your configuration. This compatibility stub will be '
+                        'removed in the L release'))
 
-    @versionutils.deprecated(
-        what='keystone.middleware.core.XmlBodyMiddleware',
-        as_of=versionutils.deprecated.ICEHOUSE,
-        in_favor_of='support for "application/json" only',
-        remove_in=+2)
     def __init__(self, *args, **kwargs):
         super(XmlBodyMiddleware, self).__init__(*args, **kwargs)
-        self.xmlns = None
-
-    def process_request(self, request):
-        """Transform the request from XML to JSON."""
-        incoming_xml = 'application/xml' in str(request.content_type)
-        if incoming_xml and request.body:
-            request.content_type = 'application/json'
-            try:
-                request.body = jsonutils.dumps(
-                    serializer.from_xml(request.body))
-            except Exception:
-                LOG.exception('Serializer failed')
-                e = exception.ValidationError(attribute='valid XML',
-                                              target='request body')
-                return wsgi.render_exception(e, request=request)
-
-    def process_response(self, request, response):
-        """Transform the response from JSON to XML."""
-        outgoing_xml = 'application/xml' in str(request.accept)
-        if outgoing_xml and response.body:
-            response.content_type = 'application/xml'
-            try:
-                body_obj = jsonutils.loads(response.body)
-                response.body = serializer.to_xml(body_obj, xmlns=self.xmlns)
-            except Exception:
-                LOG.exception('Serializer failed')
-                raise exception.Error(message=response.body)
-        return response
+        self.print_warning()
 
 
 class XmlBodyMiddlewareV2(XmlBodyMiddleware):
-    """De/serializes XML to/from JSON for v2.0 API."""
+    """De/serialize XML to/from JSON for v2.0 API."""
 
     def __init__(self, *args, **kwargs):
-        super(XmlBodyMiddlewareV2, self).__init__(*args, **kwargs)
-        self.xmlns = 'http://docs.openstack.org/identity/api/v2.0'
+        pass
 
 
 class XmlBodyMiddlewareV3(XmlBodyMiddleware):
-    """De/serializes XML to/from JSON for v3 API."""
+    """De/serialize XML to/from JSON for v3 API."""
 
     def __init__(self, *args, **kwargs):
-        super(XmlBodyMiddlewareV3, self).__init__(*args, **kwargs)
-        self.xmlns = 'http://docs.openstack.org/identity/api/v3'
+        pass
 
 
 class NormalizingFilter(wsgi.Middleware):
@@ -218,23 +185,15 @@ class NormalizingFilter(wsgi.Middleware):
             request.environ['PATH_INFO'] = '/'
 
 
-class RequestBodySizeLimiter(wsgi.Middleware):
-    """Limit the size of an incoming request."""
-
+class RequestBodySizeLimiter(sizelimit.RequestBodySizeLimiter):
+    @versionutils.deprecated(
+        versionutils.deprecated.KILO,
+        in_favor_of='oslo_middleware.sizelimit.RequestBodySizeLimiter',
+        remove_in=+1,
+        what='keystone.middleware.RequestBodySizeLimiter')
     def __init__(self, *args, **kwargs):
         super(RequestBodySizeLimiter, self).__init__(*args, **kwargs)
 
-    @webob.dec.wsgify()
-    def __call__(self, req):
-        if req.content_length is None:
-            if req.is_body_readable:
-                limiter = utils.LimitingReader(req.body_file,
-                                               CONF.max_request_body_size)
-                req.body_file = limiter
-        elif req.content_length > CONF.max_request_body_size:
-            raise exception.RequestTooLarge()
-        return self.application
-
 
 class AuthContextMiddleware(wsgi.Middleware):
     """Build the authentication context from the request auth token."""
@@ -262,7 +221,7 @@ class AuthContextMiddleware(wsgi.Middleware):
             wsgi.validate_token_bind(context, token_ref)
             return authorization.token_to_auth_context(token_ref)
         except exception.TokenNotFound:
-            LOG.warning(_('RBAC: Invalid token'))
+            LOG.warning(_LW('RBAC: Invalid token'))
             raise exception.Unauthorized()
 
     def process_request(self, request):
@@ -272,7 +231,7 @@ class AuthContextMiddleware(wsgi.Middleware):
             return
 
         if authorization.AUTH_CONTEXT_ENV in request.environ:
-            msg = _('Auth context already exists in the request environment')
+            msg = _LW('Auth context already exists in the request environment')
             LOG.warning(msg)
             return
 
